package info.aspirecn.cloud.yysj.controller;

import info.aspirecn.cloud.yysj.entity.ResponseInfo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingRequestHeaderException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 全局异常处理器
 *
 * @author dingliqiang
 */
@Slf4j
@ControllerAdvice
public class GlobalExceptionHandler {

    /**
     * 全局异常处理器
     *
     * @param e 异常信息
     */
    @ResponseBody
    @ExceptionHandler(Exception.class)
    public ResponseInfo exceptionHandler(Exception e, HttpServletResponse response) {
        log.error("", e);
        if (e instanceof Exception) {
            //response.setStatus(400);
            return new ResponseInfo().setMessage("系统异常").setData("");
        } else if (e instanceof MissingRequestHeaderException) {
            return new ResponseInfo().setMessage("Header 参数 token 不能为空").setData("");
        } else if (e instanceof MissingServletRequestParameterException){
            return new ResponseInfo().setMessage("Header 参数 token 不能为空").setData("");
        }
        response.setStatus(503);
        return new ResponseInfo().setMessage("系统繁忙，请稍后再试").setData("");
    }

    /**
     * 忽略参数异常处理器
     *
     * @param e 忽略参数异常
     * @return ResponseResult
     */
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    @ExceptionHandler(MissingServletRequestParameterException.class)
    public ResponseInfo parameterMissingExceptionHandler(MissingServletRequestParameterException e) {
        log.error("", e);
        return new ResponseInfo().setMessage("请求参数 " + e.getParameterName() + " 不能为空").setData("");
    }

    /**
     * 缺少请求体异常处理器
     *
     * @param e 缺少请求体异常
     * @return ResponseResult
     */
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    @ExceptionHandler(HttpMessageNotReadableException.class)
    public ResponseInfo parameterBodyMissingExceptionHandler(HttpMessageNotReadableException e) {
        log.error("", e);
        return new ResponseInfo().setMessage("参数体不能为空").setData("");
    }

    /**
     * 参数效验异常处理器
     *
     * @param e 参数验证异常
     * @return ResponseInfo
     */
    @ResponseStatus(HttpStatus.BAD_REQUEST)
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResponseInfo parameterExceptionHandler(MethodArgumentNotValidException e) {
        log.error("", e);
        // 获取异常信息
        BindingResult exceptions = e.getBindingResult();
        // 判断异常中是否有错误信息，如果存在就使用异常中的消息，否则使用默认消息
        if (exceptions.hasErrors()) {
            List<ObjectError> errors = exceptions.getAllErrors();
            if (!errors.isEmpty()) {
                // 这里列出了全部错误参数，按正常逻辑，只需要第一条错误即可
                FieldError fieldError = (FieldError) errors.get(0);
                return new ResponseInfo().setMessage(fieldError.getDefaultMessage()).setData("");
            }
        }
        return new ResponseInfo().setMessage("请求参数错误").setData("");
    }

}
